﻿//作成：2006 一歩二歩散歩
//テクスチャの作成と使用のサンプル

using System;
using System.ComponentModel;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using OpenGL;

namespace OpenGLTextureSample
{
    public partial class Form1 : Form
    {
        /// <summary>
        /// OpenGLのレンダリングコンテキスト
        /// </summary>
        private Context context;

        /// <summary>
        /// バックグラウンドの色
        /// </summary>
        float[] clearColor = new float[] { 0f, 0f, 0f, 1f };

        /// <summary>
        /// 描画する4角形の色
        /// </summary>
        float[] quadColor = new float[] { 1f, 1f, 1f, 1f };

        /// <summary>
        /// テクスチャ
        /// </summary>
        private Texture2D texture;

        public Form1()
        {
            InitializeComponent();

            //レンダリングコンテキストの作成・初期化
            this.context = new Context( this, 32, 16, 8 );

            //現在のレンダリングコンテキストに指定
            this.context.MakeCurrent();

            this.InitGL();

            //テクスチャの作成
            this.texture = new Texture2D( "GrassHopper.jpg" );
        }

        /// <summary>
        /// OpenGLの設定など。
        /// </summary>
        private void InitGL()
        {
            //画面クリア時に使用する色（背景色）を指定。
            gl.ClearColor( clearColor[0], clearColor[1], clearColor[2], 1.0f );

            //ビューポートの設定
            gl.Viewport( this.ClientRectangle );

            //プロジェクション行列の設定
            gl.MatrixMode( gl.PROJECTION );

            gl.LoadIdentity();

            gl.Ortho( -1, 1, -1, 1, -100, 100 );

            //テクスチャを使用可能にする。
            gl.Enable( gl.TEXTURE_2D );
        }

        /// <summary>
        /// OpenGLによる描画を行う。
        /// </summary>
        private void Draw()
        {
            //現在のレンダリングコンテキストを指定
            this.context.MakeCurrent();

            //画面のクリア            
            gl.Clear( gl.COLOR_BUFFER_BIT );

            //単純に、四角形を1つ描画する。
            gl.MatrixMode( gl.MODELVIEW );

            gl.Color3fv( quadColor );

            this.texture.Bind();

            gl.Begin( gl.QUADS );

            gl.TexCoord2f( 0f, 0f );
            gl.Vertex2f( -0.8f, -0.8f );

            gl.TexCoord2f( 1f, 0f );
            gl.Vertex2f( 0.8f, -0.8f );

            gl.TexCoord2f( 1f, 1f );
            gl.Vertex2f( 0.8f, 0.8f );

            gl.TexCoord2f( 0f, 1f );
            gl.Vertex2f( -0.8f, 0.8f );

            gl.End();

            gl.Flush();
            
            //これを行わないと画面に表示されない。
            this.context.SwapBuffers();
       }

        /// <summary>
        /// フォームの再描画を行う。
        /// </summary>
        /// <param name="e"></param>
        /// <remarks>
        /// OpPaintイベントをオーバーライドして描画する。
        ///</remarks>
        protected override void OnPaint( PaintEventArgs e )
        {
            //base.OnPaint( e );

            this.Draw();
        }

        /// <summary>
        /// フォームのサイズ変更時に発生するイベント。
        /// </summary>
        /// <param name="e"></param>
        /// <remarks>
        /// フォームのサイズが変更された時は、ビューポートを設定し直して再描画する。
        /// </remarks>
        protected override void OnResize( EventArgs e )
        {
            base.OnResize( e );

            //ビューポートの設定
            gl.Viewport( this.ClientRectangle );

            //フォームの再描画
            this.Invalidate();
        }

        private void timer1_Tick( object sender, EventArgs e )
        {
            //タイマイベントで、回転させてみる。
            gl.MatrixMode( gl.MODELVIEW );
            gl.Rotatef( 2f, 0.57735f, 0.57735f, 0.57735f );

            this.Draw();
        }
    }



}